@@ -1,10 +1,10 @@
-/* $Id: data.c,v 1.19 2002/12/15 15:00:01 dijkstra Exp $ */
+/* $Id: data.c,v 1.26 2004/08/07 12:21:36 dijkstra Exp $ */
 
 /*
- * Copyright (c) 2001-2002 Willem Dijkstra
+ * Copyright (c) 2001-2004 Willem Dijkstra
  * All rights reserved.
  *
- * The crc routine is from Rob Warnock <rpw3@sgi.com>, from the
+ * The crc routine is by Rob Warnock <rpw3@sgi.com>, from the
  * comp.compression FAQ.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,11 +35,12 @@
 /* Terminology:
  *
  * A host carrying a 'symon' is considered a 'source' of information. A single
- * data 'stream' of information has a particular type: <cpu|mem|if|io>. A
- * source can provide multiple 'streams' simultaniously. A source spools
+ * data 'stream' of information has a particular type: cpu, mem, etc. A
+ * source can provide multiple 'streams' simultaneously. A source spools
  * information towards a 'mux'. A 'stream' that has been converted to network
  * representation is called a 'packedstream'.
  */
+#include <sys/param.h>
 #include <assert.h>
 #include <limits.h>
 #include <stdarg.h>
@@ -48,6 +49,7 @@
 #include <time.h>
 #include <unistd.h>
 
+#include "conf.h"
 #include "data.h"
 #include "error.h"
 #include "lex.h"
@@ -68,6 +70,7 @@ __END_DECLS
  * Format specifications are strings of characters:
  *
  * L = u_int64
+ * D = 7.6f <= int64
  * l = u_int32
  * s = u_int16
  * c = 3.2f <= u_int14 <= u_int16  (used in percentages)
@@ -81,7 +84,8 @@ struct {
     int bytelen;
     u_int64_t max;
 }      streamvar[] = {
-    { 'L', ":%llu", " %20llu", 22, sizeof(u_int64_t), (u_int64_t) 0xffffffffffffffff },
+    { 'L', ":%llu", " %20llu", 22, sizeof(u_int64_t), (u_int64_t) 0xffffffffffffffffLL },
+    { 'D', ":%7.6f", " %7.6f", 23, sizeof(int64_t), (u_int64_t) 0xffffffffffffffffLL },
     { 'l', ":%lu", " %10lu", 12, sizeof(u_int32_t), (u_int64_t) 0xffffffff },
     { 's', ":%u", " %5u", 7, sizeof(u_int16_t), (u_int64_t) 0xffff },
     { 'c', ":%3.2f", " %3.2f", 8, sizeof(u_int16_t), (u_int64_t) 100 },
@@ -93,7 +97,7 @@ struct {
     int type;
     char *form;
 }      streamform[] = {
-    { MT_IO, "LLL" },
+    { MT_IO1, "LLL" },
     { MT_CPU, "ccccc" },
     { MT_MEM, "lllll" },
     { MT_IF, "llllllllll" },
@@ -101,6 +105,9 @@ struct {
     { MT_DEBUG, "llllllllllllllllllll" },
     { MT_PROC, "lLLLlcll" },
     { MT_MBUF, "lllllllllllllll" },
+    { MT_SENSOR, "D" },
+    { MT_IO2, "LLLLL" },
+    { MT_TEST, "LLLLDDDDllllssssccccbbbb" },
     { MT_EOT, "" }
 };
 
@@ -108,7 +115,7 @@ struct {
     int type;
     int token;
 }      streamtoken[] = {
-    { MT_IO, LXT_IO },
+    { MT_IO1, LXT_IO1 },
     { MT_CPU, LXT_CPU },
     { MT_MEM, LXT_MEM },
     { MT_IF, LXT_IF },
@@ -116,6 +123,8 @@ struct {
     { MT_DEBUG, LXT_DEBUG },
     { MT_PROC, LXT_PROC },
     { MT_MBUF, LXT_MBUF },
+    { MT_SENSOR, LXT_SENSOR },
+    { MT_IO2, LXT_IO },
     { MT_EOT, LXT_BADTOKEN }
 };
 /* parallel crc32 table */
@@ -123,7 +132,7 @@ u_int32_t
 crc32_table[256];
 
 /* Convert lexical entities to stream entities */
-const int 
+const int
 token2type(const int token)
 {
     int i;
@@ -155,7 +164,7 @@ type2str(const int streamtype)
     return 0;
 }
 /* Return the maximum lenght of the ascii representation of type <type> */
-int 
+int
 strlentype(int type)
 {
     int i = 0;
@@ -167,7 +176,7 @@ strlentype(int type)
     return sum;
 }
 /* Return the maximum lenght of the ascii representation of streamvar <var> */
-int 
+int
 strlenvar(char var)
 {
     int i;
@@ -183,7 +192,7 @@ strlenvar(char var)
     return 0;
 }
 /* Return the maximum lenght of the network representation of streamvar <var> */
-int 
+int
 bytelenvar(char var)
 {
     int i;
@@ -230,19 +239,18 @@ rrdstrvar(char var)
     return "";
 }
 /* Check whether <extra> more bytes fit in <maxlen> when we are already at <start> */
-int 
+int
 checklen(int maxlen, int current, int extra)
 {
     if ((current + extra) < maxlen) {
 	return 0;
-    }
-    else {
+    } else {
 	warning("buffer overflow: max=%d, current=%d, extra=%d",
 		maxlen, current, extra);
 	return 1;
     }
 }
-int 
+int
 setheader(char *buf, struct symonpacketheader *hph)
 {
     struct symonpacketheader nph;
@@ -266,7 +274,7 @@ setheader(char *buf, struct symonpacketheader *hph)
 
     return (p - buf);
 }
-int 
+int
 getheader(char *buf, struct symonpacketheader *hph)
 {
     char *p;
@@ -292,7 +300,7 @@ getheader(char *buf, struct symonpacketheader *hph)
  * Pack multiple arguments of a MT_TYPE into a network order bytestream.
  * snpack returns the number of bytes actually stored.
  */
-int 
+int
 snpack(char *buf, int maxlen, char *id, int type,...)
 {
     va_list ap;
@@ -301,8 +309,11 @@ snpack(char *buf, int maxlen, char *id, int type,...)
     u_int16_t c;
     u_int32_t l;
     u_int64_t q;
+    int64_t d;
+    double D;
     int i = 0;
     int offset = 0;
+    int arglen = 0;
 
     if (type > MT_EOT) {
 	warning("stream type (%d) out of range", type);
@@ -311,25 +322,26 @@ snpack(char *buf, int maxlen, char *id, int type,...)
 
     if (maxlen < 2) {
 	fatal("%s:%d: maxlen too small", __FILE__, __LINE__);
-    }
-    else {
+    } else {
 	buf[offset++] = type & 0xff;
     }
 
     if (id) {
-	if ((strlen(id) + 1) >= maxlen) {
-	    return 0;
-	}
-	else {
-	    strncpy(&buf[offset], id, maxlen - 1);
-	    offset += strlen(id);
-	}
+	arglen = MIN(strlen(id), SYMON_PS_ARGLEN - 1);
+    } else {
+	id = "\0";
+	arglen = 1;
+    }
+
+    if (checklen(maxlen, offset, arglen)) {
+	return offset;
+    } else {
+	strncpy(&buf[offset], id, arglen);
+	offset += arglen + 1;
     }
-    buf[offset++] = '\0';
 
     va_start(ap, type);
     while (streamform[type].form[i] != '\0') {
-	/* check for buffer overflow */
 	if (checklen(maxlen, offset, bytelenvar(streamform[type].form[i])))
 	    return offset;
 
@@ -337,7 +349,7 @@ snpack(char *buf, int maxlen, char *id, int type,...)
 	 * all values smaller than 32 bytes are transferred using ints on the
 	 * stack. This is to ensure that we get the correct value, if the
 	 * compiler decided to upgrade our short to a 32bit int. -- cheers
-	 * dhartmei@
+	 * dhartmei@openbsd.org
 	 */
 	switch (streamform[type].form[i]) {
 	case 'b':
@@ -346,7 +358,8 @@ snpack(char *buf, int maxlen, char *id, int type,...)
 	    break;
 
 	case 'c':
-	    c = va_arg(ap, int);
+	    D = va_arg(ap, double);
+	    c = (u_int16_t) (D * 100.0);
 	    c = htons(c);
 	    bcopy(&c, buf + offset, sizeof(u_int16_t));
 	    offset += sizeof(u_int16_t);
@@ -354,7 +367,7 @@ snpack(char *buf, int maxlen, char *id, int type,...)
 
 	case 's':
 	    s = va_arg(ap, int);
-	    s = htons(c);
+	    s = htons(s);
 	    bcopy(&s, buf + offset, sizeof(u_int16_t));
 	    offset += sizeof(u_int16_t);
 	    break;
@@ -373,8 +386,18 @@ snpack(char *buf, int maxlen, char *id, int type,...)
 	    offset += sizeof(u_int64_t);
 	    break;
 
+	case 'D':
+	    D = va_arg(ap, double);
+	    d = (int64_t) (D * 1000 * 1000);
+	    d = htonq(d);
+	    bcopy(&d, buf + offset, sizeof(int64_t));
+	    offset += sizeof(int64_t);
+	    break;
+
 	default:
-	    warning("unknown stream format identifier");
+	    warning("unknown stream format identifier %c in type %d",
+		    streamform[type].form[i],
+		    type);
 	    return 0;
 	}
 	i++;
@@ -391,7 +414,7 @@ snpack(char *buf, int maxlen, char *id, int type,...)
  * description of the packedstream (streamform) to parse the actual bytes. This
  * description corresponds to the amount of bytes that will fit inside the
  * packedstream structure.  */
-int 
+int
 sunpack(char *buf, struct packedstream * ps)
 {
     char *in, *out;
@@ -401,6 +424,7 @@ sunpack(char *buf, struct packedstream * ps)
     u_int16_t c;
     u_int32_t l;
     u_int64_t q;
+    int64_t d;
 
     bzero(ps, sizeof(struct packedstream));
 
@@ -416,14 +440,12 @@ sunpack(char *buf, struct packedstream * ps)
     if ((*in) != '\0') {
 	strncpy(ps->args, in, sizeof(ps->args));
 	ps->args[sizeof(ps->args) - 1] = '\0';
-	in += strlen(ps->args);
-    }
-    else {
+	in += strlen(ps->args) + 1;
+    } else {
 	ps->args[0] = '\0';
+	in++;
     }
 
-    in++;
-
     out = (char *) (&ps->data);
 
     while (streamform[type].form[i] != '\0') {
@@ -466,8 +488,18 @@ sunpack(char *buf, struct packedstream * ps)
 	    out += sizeof(u_int64_t);
 	    break;
 
+	case 'D':
+	    bcopy((void *) in, &d, sizeof(int64_t));
+	    d = ntohq(d);
+	    bcopy(&d, (void *) out, sizeof(int64_t));
+	    in += sizeof(int64_t);
+	    out += sizeof(int64_t);
+	    break;
+
 	default:
-	    warning("unknown stream format identifier");
+	    warning("unknown stream format identifier %c in type %d",
+		    streamform[type].form[i],
+		    type);
 	    return 0;
 	}
 	i++;
@@ -475,15 +507,16 @@ sunpack(char *buf, struct packedstream * ps)
     return (in - buf);
 }
 /* Get the RRD or 'pretty' ascii representation of packedstream */
-int 
+int
 ps2strn(struct packedstream * ps, char *buf, const int maxlen, int pretty)
 {
-    float f;
     u_int16_t b;
     u_int16_t s;
     u_int16_t c;
     u_int64_t q;
     u_int32_t l;
+    int64_t d;
+    double D;
     int i = 0;
     char *formatstr;
     char *in, *out;
@@ -518,8 +551,8 @@ ps2strn(struct packedstream * ps, char *buf, const int maxlen, int pretty)
 
 	case 'c':
 	    bcopy(in, &c, sizeof(u_int16_t));
-	    f = (float) c / 10.0;
-	    snprintf(out, strlenvar(vartype), formatstr, f);
+	    D = (double) c / 100.0;
+	    snprintf(out, strlenvar(vartype), formatstr, D);
 	    in += sizeof(u_int16_t);
 	    break;
 
@@ -541,8 +574,16 @@ ps2strn(struct packedstream * ps, char *buf, const int maxlen, int pretty)
 	    in += sizeof(u_int64_t);
 	    break;
 
+	case 'D':
+	    bcopy(in, &d, sizeof(int64_t));
+	    D = (double) (d / 1000.0 / 1000.0);
+	    snprintf(out, strlenvar(vartype), formatstr, D);
+	    in += sizeof(int64_t);
+	    break;
+
+
 	default:
-	    warning("Unknown stream format identifier");
+	    warning("unknown stream format identifier %c", vartype);
 	    return 0;
 	}
 	out += strlen(out);
@@ -573,12 +614,12 @@ find_source_stream(struct source * source, int type, char *args)
 {
     struct stream *p;
 
-    if (source == NULL)
+    if (source == NULL || args == NULL)
 	return NULL;
 
     SLIST_FOREACH(p, &source->sl, streams) {
-	if ((p->type == type)
-	    && (((void *) args != (void *) p != NULL)
+	if (((void *) p != NULL) && (p->type == type)
+	    && (((void *) args != (void *) p)
 		&& strncmp(args, p->args, _POSIX2_LINE_MAX) == 0))
 	    return p;
     }
@@ -609,12 +650,12 @@ find_mux_stream(struct mux * mux, int type, char *args)
 {
     struct stream *p;
 
-    if (mux == NULL)
+    if (mux == NULL || args == NULL)
 	return NULL;
 
     SLIST_FOREACH(p, &mux->sl, streams) {
-	if ((p->type == type)
-	    && (((void *) args != (void *) p != NULL)
+	if (((void *) p != NULL) && (p->type == type)
+	    && (((void *) args != (void *) p)
 		&& strncmp(args, p->args, _POSIX2_LINE_MAX) == 0))
 	    return p;
     }
@@ -645,11 +686,11 @@ find_source(struct sourcelist * sol, char *name)
 {
     struct source *p;
 
-    if (sol == NULL || SLIST_EMPTY(sol))
+    if (sol == NULL || SLIST_EMPTY(sol) || name == NULL)
 	return NULL;
 
     SLIST_FOREACH(p, sol, sources) {
-	if (((void *) name != (void *) p != NULL)
+	if (((void *) p != NULL) && ((void *) name != (void *) p)
 	    && strncmp(name, p->addr, _POSIX2_LINE_MAX) == 0)
 	    return p;
     }
@@ -698,11 +739,11 @@ find_mux(struct muxlist * mul, char *name)
 {
     struct mux *p;
 
-    if (mul == NULL || SLIST_EMPTY(mul))
+    if (mul == NULL || SLIST_EMPTY(mul) || name == NULL)
 	return NULL;
 
     SLIST_FOREACH(p, mul, muxes) {
-	if (((void *) name != (void *) p != NULL)
+	if (((void *) p != NULL) && ((void *) name != (void *) p)
 	    && strncmp(name, p->name, _POSIX2_LINE_MAX) == 0)
 	    return p;
     }
@@ -746,7 +787,7 @@ rename_mux(struct muxlist * mul, struct mux * mux, char *name)
 
     return mux;
 }
-void 
+void
 free_muxlist(struct muxlist * mul)
 {
     struct mux *p, *np;
@@ -780,7 +821,7 @@ free_muxlist(struct muxlist * mul)
 	p = np;
     }
 }
-void 
+void
 free_streamlist(struct streamlist * sl)
 {
     struct stream *p, *np;
@@ -802,7 +843,7 @@ free_streamlist(struct streamlist * sl)
 	p = np;
     }
 }
-void 
+void
 free_sourcelist(struct sourcelist * sol)
 {
     struct source *p, *np;
@@ -825,29 +866,28 @@ free_sourcelist(struct sourcelist * sol)
     }
 }
 /* Calculate maximum buffer space needed for a single symon hit */
-int 
+int
 calculate_churnbuffer(struct sourcelist * sol)
 {
+    char buf[_POSIX2_LINE_MAX];
     struct source *source;
     struct stream *stream;
-    int prefixlen;
     int maxlen;
     int len;
     int n;
 
-    /* determine length of a timestamp + ip as strings */
-    prefixlen = (sizeof(time_t) * 3) + strlen(":") + 15 + strlen(":");
-
     len = n = 0;
     source = NULL;
     stream = NULL;
     maxlen = 0;
+
     /* determine maximum string size for a single source */
     SLIST_FOREACH(source, sol, sources) {
-	len = prefixlen;
+	len = snprintf(&buf[0], _POSIX2_LINE_MAX, "%s;", source->addr);
 	SLIST_FOREACH(stream, &source->sl, streams) {
 	    len += strlen(type2str(stream->type)) + strlen(":");
 	    len += strlen(stream->args) + strlen(":");
+	    len += (sizeof(time_t) * 3) + strlen(":"); /* 3 > ln(255) / ln(10) */
 	    len += strlentype(stream->type);
 	    n++;
 	}
@@ -870,7 +910,7 @@ crc32(const void *buf, unsigned int len)
     return ~crc;
 }
 /* Init table for CRC32 */
-void 
+void
 init_crc32()
 {
     unsigned int i, j;
