@@ -1,7 +1,7 @@
-/* $Id: data.h,v 1.19 2003/01/08 16:04:12 dijkstra Exp $ */
+/* $Id: data.h,v 1.25 2004/03/20 15:46:27 dijkstra Exp $ */
 
 /*
- * Copyright (c) 2001-2002 Willem Dijkstra
+ * Copyright (c) 2001-2004 Willem Dijkstra
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -141,7 +141,7 @@ SLIST_HEAD(muxlist, mux);
 #define PS2STR_RRD    1
 
 /* Stream types */
-#define MT_IO     0
+#define MT_IO1    0
 #define MT_CPU    1
 #define MT_MEM    2
 #define MT_IF     3
@@ -149,17 +149,19 @@ SLIST_HEAD(muxlist, mux);
 #define MT_DEBUG  5
 #define MT_PROC   6
 #define MT_MBUF   7
-#define MT_EOT    8
+#define MT_SENSOR 8
+#define MT_IO2    9
+#define MT_TEST   10
+#define MT_EOT    11
 
 /*
  * Unpacking of incoming packets is done via a packedstream structure. This
  * structure defines the maximum amount of data that can be contained in a
- * single network representation of a stream. It is used internally for sizing
- * only. Although the union members are here, they could also read u_int64_t[4]
- * with io, for instance.
+ * single network representation of a stream.
  */
 #define SYMON_UNKMUX   "<unknown mux>"	/* mux nodes without host addr */
-#define SYMON_PS_ARGLEN    16	/* maximum argument length */
+#define SYMON_PS_ARGLEN        16	/* maximum argument length */
+#define SYMON_PS_ARGLENSTR    "15"	/* maximum number of chars in an argument, as str */
 struct packedstream {
     int type;
     int padding;
@@ -260,6 +262,24 @@ struct packedstream {
 	    u_int32_t m_wait;
 	    u_int32_t m_drain;
 	}      ps_mbuf;
+	struct {
+	    int64_t value;
+	}      ps_sensor;
+	struct {
+	    u_int64_t mtotal_rtransfers;
+	    u_int64_t mtotal_wtransfers;
+	    u_int64_t mtotal_seeks2;
+	    u_int64_t mtotal_rbytes;
+	    u_int64_t mtotal_wbytes;
+	}      ps_io2;
+	struct {
+	    u_int64_t L[4];
+	    int64_t D[4];
+	    u_int32_t l[4];
+	    u_int16_t s[4];
+	    u_int16_t c[4];
+	    u_int8_t b[4];
+	}      ps_test;
     }     data;
 };
 
