@@ -1,7 +1,7 @@
-/* $Id: lex.c,v 1.15 2002/12/15 14:27:43 dijkstra Exp $ */
+/* $Id: lex.c,v 1.24 2004/08/07 14:47:44 dijkstra Exp $ */
 
 /*
- * Copyright (c) 2001-2002 Willem Dijkstra
+ * Copyright (c) 2001-2004 Willem Dijkstra
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -49,10 +49,11 @@
 #include <sys/types.h>
 
 #include <errno.h>
+#include <fcntl.h>
 #include <limits.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 
 #include "xmalloc.h"
 #include "lex.h"
@@ -71,9 +72,12 @@ static struct {
     { "cpu", LXT_CPU },
     { "datadir", LXT_DATADIR },
     { "debug", LXT_DEBUG },
+    { "every", LXT_EVERY },
     { "if", LXT_IF },
     { "in", LXT_IN },
     { "io", LXT_IO },
+    { "io1", LXT_IO1 },
+    { "io2", LXT_IO },
     { "mbuf", LXT_MBUF },
     { "mem", LXT_MEM },
     { "monitor", LXT_MONITOR },
@@ -81,6 +85,9 @@ static struct {
     { "pf", LXT_PF },
     { "port", LXT_PORT },
     { "proc", LXT_PROC },
+    { "second", LXT_SECOND },
+    { "seconds", LXT_SECONDS },
+    { "sensor", LXT_SENSOR },
     { "source", LXT_SOURCE },
     { "stream", LXT_STREAM },
     { "to", LXT_TO },
@@ -90,7 +97,7 @@ static struct {
 #define KW_OPS "{},()"
 
 /* Return the number of the token pointed to by cp or LXT_BADTOKEN */
-int 
+int
 parse_token(const char *cp)
 {
     u_int i;
@@ -114,58 +121,54 @@ parse_opcode(const int op)
     return NULL;
 }
 /* Read a line and increase buffer if needed */
-int 
-lex_readline(struct lex * l)
+int
+lex_readline(struct lex *l)
 {
     char *bp;
 
+    if (l == NULL)
+	return 0;
+
     bp = l->buffer;
 
     if (l->buffer) {
-	if ((l->curpos < l->endpos) &&
-	    ((l->bsize - l->endpos) < _POSIX2_LINE_MAX)) {
-	    l->bsize += _POSIX2_LINE_MAX;
-	    l->buffer = xrealloc(l->buffer, l->bsize);
-	    bp = l->buffer;
-	    bp += l->endpos;
-	}
-	else {
-	    l->curpos = 0;
-	    l->endpos = 0;
-	}
-    }
-    else {
+	l->curpos = 0;
+	l->endpos = 0;
+    } else {
 	l->bsize = _POSIX2_LINE_MAX;
 	l->buffer = xmalloc(l->bsize);
 	bp = l->buffer;
     }
 
-    if (!fgets(bp, (l->buffer + l->bsize) - bp, l->fh))
-	return 0;
-    else {
-	l->endpos += strlen(bp) - 1;
-	return 1;
-    }
+    l->endpos = read(l->fh, bp, (l->buffer + l->bsize) - bp);
+
+    return (l->endpos > 0);
 }
 /* Copy char out of input stream */
-void 
-lex_copychar(struct lex * l)
+void
+lex_copychar(struct lex *l)
 {
+    if (l == NULL)
+	return;
+
     l->token[l->tokpos] = l->buffer[l->curpos];
 
     if (++l->tokpos >= _POSIX2_LINE_MAX) {
 	l->token[_POSIX2_LINE_MAX - 1] = '\0';
-	fatal("%s:%d: parse error at '%s'", l->filename, l->cline, l->token);
+	fatal("%.200s:%d: parse error at '%.200s'", l->filename, l->cline, l->token);
 	/* NOT REACHED */
     }
 }
 /* Get next char, read next line if needed */
-int 
-lex_nextchar(struct lex * l)
+int
+lex_nextchar(struct lex *l)
 {
+    if (l == NULL)
+	return 0;
+
     l->curpos++;
 
-    if (l->curpos > l->endpos)
+    if (l->curpos >= l->endpos)
 	if (!lex_readline(l))
 	    return 0;
 
@@ -175,22 +178,31 @@ lex_nextchar(struct lex * l)
     return 1;
 }
 /* Close of current token with a '\0' */
-void 
-lex_termtoken(struct lex * l)
+void
+lex_termtoken(struct lex *l)
 {
+    if (l == NULL)
+	return;
+
     l->token[l->tokpos] = l->token[_POSIX2_LINE_MAX - 1] = '\0';
     l->tokpos = 0;
 }
 /* Unget token; the lexer allows 1 look a head. */
-void 
-lex_ungettoken(struct lex * l)
+void
+lex_ungettoken(struct lex *l)
 {
+    if (l == NULL)
+	return;
+
     l->unget = 1;
 }
 /* Get the next token in lex->token. return 0 if no more tokens found. */
-int 
-lex_nexttoken(struct lex * l)
+int
+lex_nexttoken(struct lex *l)
 {
+    if (l == NULL)
+	return 0;
+
     /* return same token as last time if it has been pushed back */
     if (l->unget) {
 	l->unget = 0;
@@ -213,8 +225,7 @@ lex_nexttoken(struct lex * l)
 	    while (l->buffer[l->curpos] != '\n')
 		if (!lex_nextchar(l))
 		    return 0;
-	}
-	else if (!lex_nextchar(l))
+	} else if (!lex_nextchar(l))
 	    return 0;
     }
 
@@ -223,13 +234,13 @@ lex_nexttoken(struct lex * l)
     /* "delimited string" */
     if (l->buffer[l->curpos] == '"') {
 	if (!lex_nextchar(l)) {
-	    warning("%s:%d: unbalanced '\"'", l->filename, l->cline);
+	    warning("%.200s:%d: unbalanced '\"'", l->filename, l->cline);
 	    return 0;
 	}
 	while (l->buffer[l->curpos] != '"') {
 	    lex_copychar(l);
 	    if (!lex_nextchar(l)) {
-		warning("%s:%d: unbalanced '\"'", l->filename, l->cline);
+		warning("%.200s:%d: unbalanced '\"'", l->filename, l->cline);
 		return 0;
 	    }
 	}
@@ -241,13 +252,13 @@ lex_nexttoken(struct lex * l)
     /* 'delimited string' */
     if (l->buffer[l->curpos] == '\'') {
 	if (!lex_nextchar(l)) {
-	    warning("%s:%d: unbalanced \"\'\"", l->filename, l->cline);
+	    warning("%.200s:%d: unbalanced \"\'\"", l->filename, l->cline);
 	    return 0;
 	}
 	while (l->buffer[l->curpos] != '\'') {
 	    lex_copychar(l);
 	    if (!lex_nextchar(l)) {
-		warning("%s:%d: unbalanced \"\'\"", l->filename, l->cline);
+		warning("%.200s:%d: unbalanced \"\'\"", l->filename, l->cline);
 		return 0;
 	    }
 	}
@@ -296,36 +307,62 @@ open_lex(const char *filename)
     struct lex *l;
 
     l = xmalloc(sizeof(struct lex));
+    reset_lex(l);
     l->buffer = NULL;
-    l->cline = 1;
-    l->curpos = 0;
-    l->endpos = 0;
     l->filename = filename;
-    l->op = LXT_BADTOKEN;
     l->token = xmalloc(_POSIX2_LINE_MAX);
-    l->tokpos = 0;
-    l->type = LXY_UNKNOWN;
-    l->unget = 0;
-    l->value = 0;
 
-    if ((l->fh = fopen(l->filename, "r")) == NULL) {
-	warning("could not open file '%s':%s",
+    if ((l->fh = open(l->filename, O_RDONLY)) < 0) {
+	warning("could not open file \"%.200s\":%.200s",
 		l->filename, strerror(errno));
-	xfree(l);
+	close_lex(l);
 	return NULL;
     }
 
     lex_readline(l);
     return l;
 }
-/* Destroy a lexical analyser */
-void 
-close_lex(struct lex * l)
+/* Prepare file for another lexer run */
+void
+rewind_lex(struct lex *l)
+{
+    off_t filepos;
+
+    if (l == NULL)
+	return;
+
+    reset_lex(l);
+
+    if ((filepos = lseek(l->fh, (off_t)0, SEEK_SET)) == -1) {
+	warning("could not rewind file '%.200s':%.200s",
+		l->filename, strerror(errno));
+    }
+}
+/* Reset lexer to start of file defaults */
+void
+reset_lex(struct lex *l)
 {
     if (l == NULL)
 	return;
+
+    l->cline = 1;
+    l->curpos = 0;
+    l->endpos = 0;
+    l->op = LXT_BADTOKEN;
+    l->tokpos = 0;
+    l->type = LXY_UNKNOWN;
+    l->unget = 0;
+    l->value = 0;
+}
+/* Destroy a lexical analyser */
+void
+close_lex(struct lex *l)
+{
+    if (l == NULL)
+	return;
+
     if (l->fh)
-	fclose(l->fh);
+	close(l->fh);
     if (l->buffer)
 	xfree(l->buffer);
     if (l->token)
@@ -333,9 +370,9 @@ close_lex(struct lex * l)
     xfree(l);
 }
 /* Signal a parse error */
-void 
-parse_error(struct lex * l, const char *s)
+void
+parse_error(struct lex *l, const char *s)
 {
-    warning("%s:%d: expected %s (found '%.8s')",
+    warning("%.200s:%d: expected '%.200s' found '%.8s'",
 	    l->filename, l->cline, s, l->token);
 }
