@@ -1,7 +1,7 @@
-/* $Id: net.h,v 1.9 2002/11/29 10:45:20 dijkstra Exp $ */
+/* $Id: net.h,v 1.13 2004/08/07 12:21:36 dijkstra Exp $ */
 
 /*
- * Copyright (c) 2001-2002 Willem Dijkstra
+ * Copyright (c) 2001-2004 Willem Dijkstra
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -39,22 +39,20 @@
 
 #define SYMUX_PORT  "2100"	/* default symux port */
 
-extern char lookup_hostname[];
-extern char lookup_address[];
-extern u_int32_t lookup_ip;
 extern char res_host[];
+extern char res_service[];
 extern struct sockaddr_storage res_addr;
 
 __BEGIN_DECLS
 int cmpsock_addr(struct sockaddr *, struct sockaddr *);
+int get_numeric_name(struct sockaddr_storage *);
 int getaddr(char *, char *, int, int);
-int getip(char *);
+int getip(char *, int);
 int lookup(char *);
 void cpysock(struct sockaddr *, struct sockaddr_storage *);
 void get_inaddrany_sockaddr(struct sockaddr_storage *, int, int, char *);
 void get_mux_sockaddr(struct mux *, int);
-void get_source_sockaddr(struct source *);
+void get_source_sockaddr(struct source *, int);
 __END_DECLS
 
 #endif				/* _SYMON_LIB_NET_H */
-
