@@ -1,7 +1,7 @@
-/* $Id: symonnet.c,v 1.11 2002/11/29 10:45:21 dijkstra Exp $ */
+/* $Id: symonnet.c,v 1.14 2004/08/07 12:21:36 dijkstra Exp $ */
 
 /*
- * Copyright (c) 2001-2002 Willem Dijkstra
+ * Copyright (c) 2001-2004 Willem Dijkstra
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -39,16 +39,18 @@
 #include <errno.h>
 #include <time.h>
 
+#include "conf.h"
 #include "error.h"
 #include "data.h"
 #include "symon.h"
 #include "net.h"
 
 /* Fill a mux structure with inet details */
-void 
+void
 connect2mux(struct mux * mux)
 {
     struct sockaddr_storage sockaddr;
+
     int family;
 
     bzero((void *) &sockaddr, sizeof(sockaddr));
@@ -61,30 +63,30 @@ connect2mux(struct mux * mux)
     if ((mux->symuxsocket = socket(family, SOCK_DGRAM, 0)) == -1)
 	fatal("could not obtain socket: %.200s", strerror(errno));
 
-    if (bind(mux->symuxsocket, (struct sockaddr *) & sockaddr, sockaddr.ss_len) == -1)
+    if (bind(mux->symuxsocket, (struct sockaddr *) & sockaddr, SS_LEN(&sockaddr)) == -1)
 	fatal("could not bind socket: %.200s", strerror(errno));
 
-    info("sending packets to udp %s", mux->name);
+    info("sending packets to udp %.200s", mux->name);
 }
 /* Send data stored in the mux structure to a mux */
-void 
+void
 send_packet(struct mux * mux)
 {
     if (sendto(mux->symuxsocket, (void *) &mux->packet.data,
 	       mux->offset, 0, (struct sockaddr *) & mux->sockaddr,
-	       mux->sockaddr.ss_len)
+	       SS_LEN(&mux->sockaddr))
 	!= mux->offset) {
 	mux->senderr++;
     }
 
     if (mux->senderr >= SYMON_WARN_SENDERR) {
-	warning("%d updates to mux(%s) lost due to send errors",
+	warning("%d updates to mux(%.200s) lost due to send errors",
 		mux->senderr, mux->name);
 	mux->senderr = 0;
     }
 }
 /* Prepare a packet for data */
-void 
+void
 prepare_packet(struct mux * mux)
 {
     time_t t = time(NULL);
@@ -99,7 +101,7 @@ prepare_packet(struct mux * mux)
 		  &mux->packet.header);
 }
 /* Put a stream into the packet for a mux */
-void 
+void
 stream_in_packet(struct stream * stream, struct mux * mux)
 {
     mux->offset +=
@@ -109,7 +111,7 @@ stream_in_packet(struct stream * stream, struct mux * mux)
      stream->args);
 }
 /* Ready a packet for transmission, set length and crc */
-void 
+void
 finish_packet(struct mux * mux)
 {
     mux->packet.header.length = mux->offset;
